{-
  Uparivanje sablona (pattern matching)

  not :: Bool -> Bool
  not False = True
  not True = False

  Ako pozovemo not True uparice se prvi sablon,
  a ako pozovemo not False uparice se drugi sablon

  Dzoker (wildcard)

  Prvi nacin:
  (&&) :: Bool -> Bool -> Bool
  True && True = True
  _ && _ = False

  Moze i:
  n && m = False
  Bitno je razlicito imenovanje izraza n i m

  Ukoliko bismo zamenili redosled sablona, javila bi se greska "Pattern match overlapped"

  Drugi nacin, efikasniji:
  (&&) :: Bool -> Bool -> Bool
  True && x = x
  False && _ = False

  _ predstavlja dzoker, tj. sablon koji odgovara bilo kojoj vrednosti (obratiti paznju na redosled definisanja sablona!)

  Sabloni lista:

  length [_] = 1 - [_] predstavlja listu sa jednim elementom
  length x = 1 + length (tail x) - x predstavlja listu

-}

length1 [] = 0
length1 x = 1 + length1 (tail x)

-- (x:xs) - predstavlja sablon liste, x je glava, xs je rep

length2 [] = 0
length2 (_:xs) = 1 + length2 xs

-- preslikava iz liste elemenata u jedan element
glava :: [a] -> a
glava (x:_) = x

-- preslikava iz liste u listu
rep :: [a] -> [a]
rep (_:xs) = xs

-- restrikcija na liste elemenata koji pripadaju numerickim tipovima
glavaNum :: (Num a) => [a] -> a
glavaNum (x:_) = x

-- Generatori liste (list comprenhension)

-- even proverava da li je argument paran, a odd da li je neparan
-- elementi listi pripadaju skupu od 1 do 50
parni = [x | x<-[1..50], even x]
neparni = [x | x<-[1..50], odd x]

deljiviPet = [x | x<-[1..50], mod x 5 == 0] -- elementi liste pripadaju skupu od 1 do 50 i deljivi su sa 5

kvadrati = [x^2 | x<-[1..10]]

-- Mozemo imati vise generatora

torke1 = [(x,y) | x<-[1..5], y<-[6..9]]

-- obratiti paznju kako utice zamena mesta generatora
torke2 = [(x,y) | y<-[6..9], x<-[1..5]]

-- sa negativnim korakom
torke3 = [(x,y) | y<-[9,8..6], x<-[1..5]]

-- Generatori mogu zavisiti jedni od drugih

-- kasniji generatori mogu zavisiti samo od promenljivih koje su uvedene ranijim generatorima citajuci sa leva na desno
torke4 = [(x,y) | x<-[1..10], y<-[x..10]]

{-
  Sekcija where

  where ime = vrednost
  - imena koja definisemo u where sekciji su vidljiva samo u okviru funkcije
  u kojoj su definisana
-}

bmiIzracunaj tezina visina
    | bmi <= 18.5 = "Mrsavko"
    | bmi <= 25.0 = "Sportista"
    | bmi > 25.0    = "Bucko"
    where bmi = tezina / visina ^ 2
